<?php

namespace Drupal\ris;

use Drupal\Component\Datetime\TimeInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\File\Exception\FileException;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\file\FileInterface;
use Drupal\file\FileRepositoryInterface;
use Drupal\image\ImageStyleInterface;
use GuzzleHttp\Client as HttpClient;
use GuzzleHttp\RequestOptions;
use Psr\Log\LoggerInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpKernel\Event\ControllerEvent;
use Symfony\Component\HttpKernel\KernelEvents;

/**
 * Provides base operation methods for the Remote Image Styles module.
 */
class RisService implements EventSubscriberInterface {

  const FILES_SUBDIR = 'ris';

  /**
   * Logger channel object.
   */
  protected LoggerInterface $logger;

  public function __construct(
    protected ConfigFactoryInterface $configFactory,
    protected EntityTypeManagerInterface $entityTypeManager,
    protected FileRepositoryInterface $fileRepository,
    protected FileSystemInterface $fileSystem,
    protected HttpClient $httpClient,
    LoggerChannelFactoryInterface $loggerChannelFactory,
    protected TimeInterface $time,
  ) {
    $this->logger = $loggerChannelFactory->get('ris');
  }

  /**
   * Checks whether uri is remote.
   */
  public function isRemoteUri(string $uri): bool {
    return str_starts_with($uri, 'https://');
  }

  /**
   * Returns file entity by uri if it's remote.
   */
  public function getRemoteFile(string $uri): ?FileInterface {
    return $this->isRemoteUri($uri)
      ? $this->fileRepository->loadByUri($uri)
      : NULL;
  }

  /**
   * Builds a local uri in a dedicated directory for a file.
   */
  public function buildLocalUri(FileInterface $file): string {
    if (!$file->id()) {
      $file->save();
    }

    return sprintf(
      'public://%s/%s-%s',
      static::FILES_SUBDIR,
      $file->id(),
      $file->getFilename(),
    );
  }

  /**
   * Downloads image to local filesystem.
   *
   * @param \Drupal\file\FileInterface $file
   *   File entity.
   *
   * @return string
   *   Local uri.
   *
   * @throws \Drupal\Core\File\Exception\FileException
   * @throws \Drupal\ris\RisException
   * @throws \GuzzleHttp\Exception\GuzzleException
   */
  public function download(FileInterface $file): string {
    $source = $file->getFileUri();

    if (!$source || !$this->isRemoteUri($source)) {
      throw new RisException('Unsupported uri ' . $source);
    }

    $destination = $this->buildLocalUri($file);

    if (!file_exists($destination)) {
      $directory = $this->fileSystem->dirname($destination);

      if (!$this->fileSystem->prepareDirectory($directory, $this->fileSystem::CREATE_DIRECTORY | $this->fileSystem::MODIFY_PERMISSIONS)) {
        throw new RisException('Unable to prepare directory ' . $directory);
      }

      // Set low timeout value - there may be a lot of styled images on a page.
      $response = $this->httpClient->get($source, [RequestOptions::TIMEOUT => 5]);
      $this->fileSystem->saveData((string) $response->getBody(), $destination);
    }

    return $destination;
  }

  /**
   * Replaces the remote image uri with a local one.
   *
   * @see image_theme()
   * @see template_preprocess_image_style()
   */
  public function preprocessImageStyle(array &$variables): void {
    $uri = $variables['uri'];

    if ($file = $this->getRemoteFile($uri)) {
      /** @var \Drupal\image\ImageStyleInterface $style */
      $style = $this->entityTypeManager->getStorage('image_style')
        ->load($variables['style_name']);

      // If style does not support file type, original image is rendered.
      if ($style && $style->supportsUri($uri)) {
        $uri = $this->buildLocalUri($file);

        $variables['uri'] = $uri;
        $variables['image']['#uri'] = $style->buildUrl($uri);
      }
    }
  }

  /**
   * Deletes an image derivative of a remote image.
   *
   * @see \Drupal\image\Entity\ImageStyle::flush()
   */
  public function imageStyleFlush(ImageStyleInterface $style, ?string $path = NULL): void {
    if (isset($path) && $file = $this->getRemoteFile($path)) {
      $derivative_uri = $style->buildUri($this->buildLocalUri($file));

      try {
        $this->fileSystem->delete($derivative_uri);
      }
      catch (FileException) {
      }
    }
  }

  /**
   * Deletes a remote image copy from the local filesystem.
   */
  public function filePredelete(FileInterface $file): void {
    $uri = $file->getFileUri();

    if ($uri && $this->isRemoteUri($uri)) {
      $uri = $this->buildLocalUri($file);

      try {
        $this->fileSystem->delete($uri);
      }
      catch (FileException) {
      }
    }
  }

  /**
   * Deletes old remote image copies from the local filesystem.
   *
   * @see file_cron()
   */
  public function cron(): void {
    $directory = 'public://' . static::FILES_SUBDIR;
    $age = $this->configFactory->get('system.file')->get('temporary_maximum_age');

    if ($age && is_dir($directory)) {
      foreach (array_keys($this->fileSystem->scanDirectory($directory, '#.+#')) as $uri) {
        if (filemtime($uri) < $this->time->getRequestTime() - $age) {
          $this->fileSystem->delete($uri);
        }
      }
    }
  }

  /**
   * Downloads image to local filesystem for the image style controller.
   *
   * @param \Symfony\Component\HttpKernel\Event\ControllerEvent $event
   *   Event object.
   *
   * @see \Drupal\Core\Routing\UrlMatcher::getAttributes()
   * @see \Drupal\image\Routing\ImageStyleRoutes::routes()
   * @see \Drupal\image\PathProcessor\PathProcessorImageStyles::processInbound()
   */
  public function onControllerDownload(ControllerEvent $event): void {
    $request = $event->getRequest();

    if ($request->attributes->get('_route') !== 'image.style_public') {
      return;
    }

    $scheme = $request->attributes->get('_route_params')['scheme'] ?? NULL;
    $target = $request->query->get('file', '');

    $pattern = '#^' . preg_quote(static::FILES_SUBDIR, '#') . '/(\d+)-[^/]+$#';

    if ($scheme !== 'public' || !preg_match($pattern, $target, $match)) {
      return;
    }

    /** @var \Drupal\file\FileInterface $file */
    if ($file = $this->entityTypeManager->getStorage('file')->load($match[1])) {
      try {
        $this->download($file);
      }
      // GuzzleException is not an instance of \Exception.
      catch (\Throwable $e) {
        $this->logger->error('Unable to copy file @source to @destination due to error "%error".', [
          '@source' => $file->getFileUri(),
          '@destination' => $this->buildLocalUri($file),
          '%error' => $e->getMessage(),
        ]);
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents(): array {
    $events[KernelEvents::CONTROLLER][] = ['onControllerDownload'];
    return $events;
  }

}
