<?php

declare(strict_types=1);

namespace Drupal\Tests\ris\Kernel;

use Drupal\Core\File\FileSystemInterface;
use Drupal\file\Entity\File;
use Drupal\image\Entity\ImageStyle;
use Drupal\KernelTests\KernelTestBase;

/**
 * Tests ris module hook implementations.
 *
 * @group ris
 */
class HooksTest extends KernelTestBase {

  /**
   * File ID used by tests.
   */
  private int $fileId = 100;

  /**
   * File uri used by tests.
   */
  private string $fileUri = 'https://host/img.jpg';

  /**
   * File name used by tests.
   */
  private string $fileName = 'filename.jpeg';

  /**
   * Image style ID used by tests.
   */
  private string $imageStyleId = 'thumbnail';

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'ris',
    'file',
    'image',
    'system',
    'user',
  ];

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();
    $this->installEntitySchema('user');
    $this->installEntitySchema('file');
    $this->installSchema('file', 'file_usage');
    $this->installConfig(['file', 'image', 'system']);

    File::create([
      'fid' => $this->fileId,
      'uri' => $this->fileUri,
      'filename' => $this->fileName,
    ])->save();
  }

  /**
   * Creates a test file.
   */
  protected function prepareFile(string $uri): void {
    $dir = dirname($uri);
    \Drupal::service('file_system')->prepareDirectory(
      $dir,
      FileSystemInterface::CREATE_DIRECTORY | FileSystemInterface::MODIFY_PERMISSIONS,
    );
    file_put_contents($uri, 'test');
    $this->assertFileExists($uri);
  }

  /**
   * Tests the preprocessing of an image style.
   */
  public function testHookPreprocessImageStyle(): void {
    $image_style = [
      '#theme' => 'image_style',
      '#uri' => $this->fileUri,
      '#width' => 40,
      '#height' => 20,
      '#style_name' => $this->imageStyleId,
    ];
    $output = (string) \Drupal::service('renderer')->renderRoot($image_style);
    $this->assertMatchesRegularExpression(
      '~src="[^"]+/styles/' . $this->imageStyleId . '/public/ris/' . $this->fileId . '-' . preg_quote($this->fileName, '~') . '\.webp\?itok=~',
      $output,
    );
  }

  /**
   * Tests the image style flush hook.
   */
  public function testHookImageStyleFlush(): void {
    $file = "public://styles/{$this->imageStyleId}/public/ris/{$this->fileId}-{$this->fileName}.webp";
    $this->prepareFile($file);

    ImageStyle::load($this->imageStyleId)->flush($this->fileUri);
    $this->assertFileDoesNotExist($file);
  }

  /**
   * Tests the predelete hook on the file entity.
   */
  public function testHookFilePredelete(): void {
    $file = "public://ris/{$this->fileId}-{$this->fileName}";
    $this->prepareFile($file);

    File::load($this->fileId)->delete();
    $this->assertFileDoesNotExist($file);
  }

  /**
   * Tests the cron hook.
   */
  public function testHookCron(): void {
    $request_time = \Drupal::time()->getRequestTime();
    $temp_max_age = \Drupal::config('system.file')->get('temporary_maximum_age');
    $dir = 'public://ris';
    $this->prepareFile("$dir/old");
    $this->prepareFile("$dir/fresh");
    touch("$dir/old", $request_time - $temp_max_age - 1);
    touch("$dir/fresh", $request_time - $temp_max_age);

    \Drupal::service('cron')->run();
    $this->assertFileDoesNotExist("$dir/old");
    $this->assertFileExists("$dir/fresh");
  }

}
