<?php

declare(strict_types=1);

namespace Drupal\Tests\ris\Kernel;

use Drupal\file\Entity\File;
use Drupal\KernelTests\KernelTestBase;
use Drupal\ris\RisException;
use Drupal\ris\RisService;
use Drupal\Tests\ris\Traits\CommonThingsTrait;

/**
 * Tests the image downloading.
 *
 * @group ris
 */
class RisServiceDownloadTest extends KernelTestBase {

  use CommonThingsTrait;

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'ris',
    'file',
    'user',
  ];

  /**
   * Tests the download() method with a file with an unsupported uri.
   *
   * @dataProvider providerDownloadUnsupportedUri
   */
  public function testUnsupportedUri(array $file_values): void {
    $ris = \Drupal::service(RisService::class);
    $file = File::create($file_values);

    $this->expectException(RisException::class);
    $this->expectExceptionMessageMatches('/^Unsupported uri/');
    $ris->download($file);
  }

  /**
   * Provides data and expected results for the test method.
   *
   * @return array
   *   Data and expected results.
   */
  public static function providerDownloadUnsupportedUri(): array {
    return [
      [[]],
      [['uri' => 'public://test']],
    ];
  }

  /**
   * Tests the download() method with existing destination file.
   */
  public function testFileExists(): void {
    $ris = \Drupal::service(RisService::class);
    $file = File::create([
      'fid' => $fid = rand(),
      'uri' => 'https://non-existing-uri',
      'filename' => $name = $this->randomMachineName(),
    ]);

    mkdir('public://ris');
    file_put_contents($path = 'public://ris/' . $fid . '-' . $name, '');

    $this->assertSame($path, $ris->download($file));
  }

  /**
   * Tests the download() method with inability to prepare the directory.
   */
  public function testUnableToPrepareDir(): void {
    $ris = \Drupal::service(RisService::class);
    $file = File::create([
      'fid' => rand(),
      'uri' => 'https://test',
      'filename' => $this->randomMachineName(),
    ]);

    // Create a file in place of directory.
    file_put_contents('public://ris', '');

    $this->expectException(RisException::class);
    $this->expectExceptionMessage('Unable to prepare directory public://ris');
    $ris->download($file);
  }

  /**
   * Tests the download() method.
   */
  public function testNormal(): void {
    $ris = \Drupal::service(RisService::class);
    $file = File::create([
      'fid' => $fid = rand(),
      'uri' => $this->remoteImageUrl,
      'filename' => $name = $this->randomMachineName(),
    ]);

    $path = 'public://ris/' . $fid . '-' . $name;
    $this->assertSame($path, $ris->download($file));
    $this->assertFileExists($path);
    $this->assertSame(file_get_contents($this->remoteImageUrl), file_get_contents($path));
  }

}
