<?php

declare(strict_types=1);

namespace Drupal\Tests\ris\Unit;

use Drupal\Core\Image\Image;
use Drupal\Core\Image\ImageFactory;
use Drupal\Core\ImageToolkit\ImageToolkitInterface;
use Drupal\Core\ImageToolkit\ImageToolkitManager;
use Drupal\file\FileInterface;
use Drupal\ris\RemoteImage;
use Drupal\ris\RisImageFactory;
use Drupal\ris\RisService;
use Drupal\Tests\UnitTestCase;

/**
 * Tests RisImageFactory methods.
 *
 * @coversDefaultClass \Drupal\ris\RisImageFactory
 * @group ris
 */
class RisImageFactoryTest extends UnitTestCase {

  /**
   * RIS image factory service.
   */
  protected RisImageFactory $risImageFactory;

  /**
   * Mocked decorated image factory.
   */
  protected ImageFactory $decoratedImageFactory;

  /**
   * Mocked RIS service.
   */
  protected RisService $risService;

  /**
   * Mocked image toolkit plugin manager.
   */
  protected ImageToolkitManager $imageToolkitManager;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->risImageFactory = new RisImageFactory(
      $this->decoratedImageFactory = $this->createMock(ImageFactory::class),
      $this->risService = $this->createMock(RisService::class),
      $this->imageToolkitManager = $this->createMock(ImageToolkitManager::class),
    );
  }

  /**
   * @covers ::get
   * @covers ::doGet
   * @dataProvider providerGet
   */
  public function testGet($is_image_valid): void {
    $source = $this->randomMachineName(8);
    $toolkit_id = $this->randomMachineName();
    $file = $this->createMock(FileInterface::class);

    $toolkit = $this->createMock(ImageToolkitInterface::class);
    $toolkit->expects($this->any())
      ->method('setSource')
      ->willReturnSelf();
    $toolkit->expects($this->any())
      ->method('parseFile');
    $toolkit->expects($this->once())
      ->method('isValid')
      ->willReturn($is_image_valid);

    $this->risService->expects($this->once())
      ->method('getRemoteFile')
      ->willReturn($file);
    $this->imageToolkitManager->expects($this->any())
      ->method('createInstance')
      ->willReturn($toolkit);

    if (!$is_image_valid) {
      $local_source = $this->randomMachineName(9);

      $this->risService->expects($this->once())
        ->method('download')
        ->with($file)
        ->willReturn($local_source);
    }

    $image = $this->risImageFactory->get($source, $toolkit_id);
    $this->assertSame(RemoteImage::class, $image::class);
    $this->assertSame($is_image_valid ? $source : $local_source, $image->getSource());
  }

  /**
   * Provides data and expected results for the test method.
   *
   * @return array
   *   Data and expected results.
   */
  public static function providerGet(): array {
    return [[TRUE], [FALSE]];
  }

  /**
   * @covers ::get
   */
  public function testGetPassingToInnerWithoutSource(): void {
    $image = $this->createMock(Image::class);

    $this->decoratedImageFactory->expects($this->once())
      ->method('get')
      ->with(NULL, NULL)
      ->willReturn($image);

    $this->risImageFactory->get();
  }

  /**
   * @covers ::get
   */
  public function testGetPassingToInnerNotRemoteFile(): void {
    $source = $this->randomMachineName();
    $image = $this->createMock(Image::class);

    $this->risService->expects($this->once())
      ->method('getRemoteFile')
      ->willReturn(NULL);
    $this->decoratedImageFactory->expects($this->once())
      ->method('get')
      ->with($source, NULL)
      ->willReturn($image);

    $this->risImageFactory->get($source);
  }

  /**
   * @covers ::setToolkitId
   */
  public function testSetToolkitId(): void {
    $toolkit_id = $this->randomMachineName();

    $this->decoratedImageFactory->expects($this->once())
      ->method('setToolkitId')
      ->with($toolkit_id);

    $result = $this->risImageFactory->setToolkitId($toolkit_id);
    $this->assertSame($this->risImageFactory, $result);
  }

  /**
   * @covers ::getToolkitId
   */
  public function testGetToolkitId(): void {
    $toolkit_id = $this->randomMachineName();

    $this->decoratedImageFactory->expects($this->once())
      ->method('getToolkitId')
      ->willReturn($toolkit_id);

    $this->assertSame($toolkit_id, $this->risImageFactory->getToolkitId());
  }

  /**
   * @covers ::getSupportedExtensions
   */
  public function testGetSupportedExtensions(): void {
    $toolkit_id = $this->randomMachineName();
    $extensions = array_values((array) $this->randomObject());

    $this->decoratedImageFactory->expects($this->once())
      ->method('getSupportedExtensions')
      ->with($toolkit_id)
      ->willReturn($extensions);

    $this->assertSame($extensions, $this->risImageFactory->getSupportedExtensions($toolkit_id));
  }

}
