<?php

declare(strict_types=1);

namespace Drupal\Tests\ris\Unit;

use Drupal\Component\Datetime\TimeInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\file\FileInterface;
use Drupal\file\FileRepositoryInterface;
use Drupal\ris\RisService;
use Drupal\Tests\UnitTestCase;
use GuzzleHttp\Client;
use Psr\Log\LoggerInterface;

/**
 * Tests RisService methods.
 *
 * @coversDefaultClass \Drupal\ris\RisService
 * @group ris
 */
class RisServiceTest extends UnitTestCase {

  /**
   * RIS service.
   */
  protected RisService $risService;

  /**
   * Mocked file repository.
   */
  protected FileRepositoryInterface $fileRepository;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $logger = $this->createMock(LoggerInterface::class);
    $loggerChannelFactory = $this->createMock(LoggerChannelFactoryInterface::class);
    $loggerChannelFactory->expects($this->any())
      ->method('get')
      ->with('ris')
      ->willReturn($logger);

    $this->risService = new RisService(
      $this->createMock(ConfigFactoryInterface::class),
      $this->createMock(EntityTypeManagerInterface::class),
      $this->fileRepository = $this->createMock(FileRepositoryInterface::class),
      $this->createMock(FileSystemInterface::class),
      $this->createMock(Client::class),
      $loggerChannelFactory,
      $this->createMock(TimeInterface::class),
    );
  }

  /**
   * @covers ::isRemoteUri
   * @dataProvider providerIsRemoteUri
   */
  public function testIsRemoteUri($expected_result, $uri): void {
    $this->assertSame($expected_result, $this->risService->isRemoteUri($uri));
  }

  /**
   * Provides data and expected results for the test method.
   *
   * @return array
   *   Data and expected results.
   */
  public static function providerIsRemoteUri(): array {
    return [
      [FALSE, 'http://example.com'],
      [TRUE, 'https://drupal.org'],
      [FALSE, 'public://file'],
      [FALSE, 'path'],
    ];
  }

  /**
   * @covers ::getRemoteFile
   * @dataProvider providerGetRemoteFile
   */
  public function testGetRemoteFile($uri, $file): void {
    if ($file === TRUE) {
      $file = $this->createMock(FileInterface::class);
    }

    $this->fileRepository->expects($this->any())
      ->method('loadByUri')
      ->with($uri)
      ->willReturn($file);

    $this->assertSame($file, $this->risService->getRemoteFile($uri));
  }

  /**
   * Provides data and expected results for the test method.
   *
   * @return array
   *   Data and expected results.
   */
  public static function providerGetRemoteFile(): array {
    return [
      ['public', NULL],
      ['https://', NULL],
      ['https://', TRUE],
    ];
  }

  /**
   * @covers ::buildLocalUri
   * @dataProvider providerBuildLocalUri
   */
  public function testBuildLocalUri($fid1, $fid2, $filename, $result): void {
    $file = $this->createMock(FileInterface::class);

    $file->expects($this->exactly(2))
      ->method('id')
      ->willReturn($fid1)
      ->willReturn($fid2);

    $file->expects($this->any())
      ->method('save');
    $file->expects($this->once())
      ->method('getFilename')
      ->willReturn($filename);

    $this->assertSame($result, $this->risService->buildLocalUri($file));
  }

  /**
   * Provides data and expected results for the test method.
   *
   * @return array
   *   Data and expected results.
   */
  public static function providerBuildLocalUri(): array {
    return [
      [NULL, 12, 'abc', 'public://ris/12-abc'],
      [23, 23, 'xyz', 'public://ris/23-xyz'],
    ];
  }

}
